WORKSPACE_ENDPOINT = "/workspace/"
SINGLE_WORKSPACE_ENDPOINT = WORKSPACE_ENDPOINT + "{id}"
WORKSPACE_APPS_ENDPOINT = SINGLE_WORKSPACE_ENDPOINT + "/apps"
WORKSPACE_MODELS_ENDPOINT = SINGLE_WORKSPACE_ENDPOINT + "/models"
WORKSPACE_CONTEXTS_ENDPOINT = SINGLE_WORKSPACE_ENDPOINT + "/context"

DEFAULT_WORKSPACE_PAGE_SIZE = 50
