CONTEXT_ENDPOINT = "/context/"
SINGLE_CONTEXT_ENDPOINT = CONTEXT_ENDPOINT + "{id}"
GENERATE_EMBEDDINGS = f"{CONTEXT_ENDPOINT}embed"
UPLOAD_PRE_SIGNED_URL_ENDPOINT = f"{CONTEXT_ENDPOINT}presign"
CONTEXT_STATUS_ENDPOINT = f"{SINGLE_CONTEXT_ENDPOINT}/status"
CONTEXT_SEARCH_ENDPOINT = f"{SINGLE_CONTEXT_ENDPOINT}/search"
CONTEXT_PROMPT_ENDPOINT = f"{CONTEXT_ENDPOINT}prompt"
CONTEXT_DOCUMENTS_ENDPOINT = f"{SINGLE_CONTEXT_ENDPOINT}/documents"
UPDATE_CONTEXT_DOCUMENT_ENDPOINT = f"{SINGLE_CONTEXT_ENDPOINT}/documents/" + "{doc_id}"
