ACTION_ENDPOINT = "/actions/"
CREATE_ACTION_ENDPOINT = f"{ACTION_ENDPOINT}create"
SINGLE_ACTION_ENDPOINT = ACTION_ENDPOINT + "{id}"
ACTION_DATA_ENDPOINT = f"{SINGLE_ACTION_ENDPOINT}/data"
PLAYGROUND_PROMPT_ENDPOINT = f"{ACTION_ENDPOINT}playground"
DEPLOY_ACTION_ENDPOINT = f"{SINGLE_ACTION_ENDPOINT}/deploy"
ACTION_SKILL_ENDPOINT = f"{SINGLE_ACTION_ENDPOINT}/skills"
ACTION_CONTEXT_ENDPOINT = f"{SINGLE_ACTION_ENDPOINT}/context"

DEFAULT_ACTIONS_PAGE_SIZE = 50
