APP_ENDPOINT = "/apps/"
SINGLE_APP_ENDPOINT = APP_ENDPOINT + "{id}"
APP_DATA_ENDPOINT = f"{SINGLE_APP_ENDPOINT}/data"
APP_ACTIONS_ENDPOINT = f"{SINGLE_APP_ENDPOINT}/actions"

DEFAULT_APPS_PAGE_SIZE = 50
